@@ -13,16 +13,8 @@
 #include "pci_regs.h" // PCI_ROM_ADDRESS
 #include "pci_ids.h" // PCI_CLASS_DISPLAY_VGA
 #include "boot.h" // IPL
-#include "paravirt.h" // qemu_cfg_*
 #include "optionroms.h" // struct rom_header
 
-/****************************************************************
- * Definitions
- ****************************************************************/
-
-// The end of the last deployed rom.
-u32 RomEnd = BUILD_ROM_START;
-
 
 /****************************************************************
  * Helper functions
@@ -45,7 +37,7 @@ __callrom(struct rom_header *rom, u16 offset, u16 bdf)
     br.di = get_pnp_offset();
     br.code = SEGOFF(seg, offset);
     start_preempt();
-    call16big(&br);
+    farcall16big(&br);
     finish_preempt();
 
     debug_serial_setup();
@@ -120,47 +112,25 @@ get_pci_rom(struct rom_header *rom)
     return pd;
 }
 
-// Return start of code in 0xc0000-0xf0000 space.
-static inline u32 _max_rom(void) {
-    extern u8 code32flat_start[], code32init_end[];
-    return CONFIG_RELOCATE_INIT ? (u32)code32init_end : (u32)code32flat_start;
-}
-// Return the memory position up to which roms may be located.
-static inline u32 max_rom(void) {
-    u32 end = _max_rom();
-    return end > BUILD_BIOS_ADDR ? BUILD_BIOS_ADDR : end;
-}
-
-// Copy a rom to its permanent location below 1MiB
-static struct rom_header *
-copy_rom(struct rom_header *rom)
-{
-    u32 romsize = rom->size * 512;
-    if (RomEnd + romsize > max_rom()) {
-        // Option rom doesn't fit.
-        warn_noalloc();
-        return NULL;
-    }
-    dprintf(4, "Copying option rom (size %d) from %p to %x\n"
-            , romsize, rom, RomEnd);
-    iomemcpy((void*)RomEnd, rom, romsize);
-    return (void*)RomEnd;
-}
-
 // Run rom init code and note rom size.
 static int
 init_optionrom(struct rom_header *rom, u16 bdf, int isvga)
 {
     if (! is_valid_rom(rom))
         return -1;
+    struct rom_header *newrom = rom_reserve(rom->size * 512);
+    if (!newrom) {
+        warn_noalloc();
+        return -1;
+    }
+    if (newrom != rom)
+        memmove(newrom, rom, rom->size * 512);
 
-    if (isvga || get_pnp_rom(rom))
+    if (isvga || get_pnp_rom(newrom))
         // Only init vga and PnP roms here.
-        callrom(rom, bdf);
+        callrom(newrom, bdf);
 
-    RomEnd = (u32)rom + ALIGN(rom->size * 512, OPTION_ROM_ALIGN);
-
-    return 0;
+    return rom_confirm(newrom->size * 512);
 }
 
 #define RS_PCIROM (1LL<<33)
@@ -180,7 +150,8 @@ getRomPriority(u64 *sources, struct rom_header *rom, int instance)
         return -1;
     if (source & RS_PCIROM)
         return bootprio_find_pci_rom((void*)(u32)source, instance);
-    return bootprio_find_named_rom(romfile_name(source), instance);
+    struct romfile_s *file = (void*)(u32)source;
+    return bootprio_find_named_rom(file->name, instance);
 }
 
 
@@ -188,6 +159,21 @@ getRomPriority(u64 *sources, struct rom_header *rom, int instance)
  * Roms in CBFS
  ****************************************************************/
 
+static struct rom_header *
+deploy_romfile(struct romfile_s *file)
+{
+    u32 size = file->size;
+    struct rom_header *rom = rom_reserve(size);
+    if (!rom) {
+        warn_noalloc();
+        return NULL;
+    }
+    int ret = file->copy(file, rom, size);
+    if (ret <= 0)
+        return NULL;
+    return rom;
+}
+
 // Check if an option rom is at a hardcoded location or in CBFS.
 static struct rom_header *
 lookup_hardcode(struct pci_device *pci)
@@ -195,26 +181,24 @@ lookup_hardcode(struct pci_device *pci)
     char fname[17];
     snprintf(fname, sizeof(fname), "pci%04x,%04x.rom"
              , pci->vendor, pci->device);
-    int ret = romfile_copy(romfile_find(fname), (void*)RomEnd
-                           , max_rom() - RomEnd);
-    if (ret <= 0)
-        return NULL;
-    return (void*)RomEnd;
+    struct romfile_s *file = romfile_find(fname);
+    if (file)
+        return deploy_romfile(file);
+    return NULL;
 }
 
 // Run all roms in a given CBFS directory.
 static void
 run_file_roms(const char *prefix, int isvga, u64 *sources)
 {
-    u32 file = 0;
+    struct romfile_s *file = NULL;
     for (;;) {
         file = romfile_findprefix(prefix, file);
         if (!file)
             break;
-        struct rom_header *rom = (void*)RomEnd;
-        int ret = romfile_copy(file, rom, max_rom() - RomEnd);
-        if (ret > 0) {
-            setRomSource(sources, rom, file);
+        struct rom_header *rom = deploy_romfile(file);
+        if (rom) {
+            setRomSource(sources, rom, (u32)file);
             init_optionrom(rom, 0, isvga);
         }
     }
@@ -243,6 +227,22 @@ is_pci_vga(struct pci_device *pci)
     return 1;
 }
 
+// Copy a rom to its permanent location below 1MiB
+static struct rom_header *
+copy_rom(struct rom_header *rom)
+{
+    u32 romsize = rom->size * 512;
+    struct rom_header *newrom = rom_reserve(romsize);
+    if (!newrom) {
+        warn_noalloc();
+        return NULL;
+    }
+    dprintf(4, "Copying option rom (size %d) from %p to %p\n"
+            , romsize, rom, newrom);
+    iomemcpy(newrom, rom, romsize);
+    return newrom;
+}
+
 // Map the option rom of a given PCI device.
 static struct rom_header *
 map_pcirom(struct pci_device *pci)
@@ -346,17 +346,17 @@ optionrom_setup(void)
     dprintf(1, "Scan for option roms\n");
     u64 sources[(BUILD_BIOS_ADDR - BUILD_ROM_START) / OPTION_ROM_ALIGN];
     memset(sources, 0, sizeof(sources));
-    u32 post_vga = RomEnd;
+    u32 post_vga = rom_get_last();
 
     if (CONFIG_OPTIONROMS_DEPLOYED) {
         // Option roms are already deployed on the system.
-        u32 pos = RomEnd;
-        while (pos < max_rom()) {
+        u32 pos = post_vga;
+        while (pos < rom_get_top()) {
             int ret = init_optionrom((void*)pos, 0, 0);
             if (ret)
                 pos += OPTION_ROM_ALIGN;
             else
-                pos = RomEnd;
+                pos = rom_get_last();
         }
     } else {
         // Find and deploy PCI roms.
@@ -370,11 +370,12 @@ optionrom_setup(void)
         // Find and deploy CBFS roms not associated with a device.
         run_file_roms("genroms/", 0, sources);
     }
+    rom_reserve(0);
 
     // All option roms found and deployed - now build BEV/BCV vectors.
 
     u32 pos = post_vga;
-    while (pos < RomEnd) {
+    while (pos < rom_get_last()) {
         struct rom_header *rom = (void*)pos;
         if (! is_valid_rom(rom)) {
             pos += OPTION_ROM_ALIGN;
@@ -411,6 +412,7 @@ optionrom_setup(void)
 
 static int S3ResumeVgaInit;
 int ScreenAndDebug;
+struct rom_header *VgaROM;
 
 // Call into vga code to turn on console.
 void
@@ -431,7 +433,7 @@ vga_setup(void)
         init_optionrom((void*)BUILD_ROM_START, 0, 1);
     } else {
         // Clear option rom memory
-        memset((void*)RomEnd, 0, max_rom() - RomEnd);
+        memset((void*)BUILD_ROM_START, 0, rom_get_top() - BUILD_ROM_START);
 
         // Find and deploy PCI VGA rom.
         struct pci_device *pci;
@@ -446,13 +448,13 @@ vga_setup(void)
         // Find and deploy CBFS vga-style roms not associated with a device.
         run_file_roms("vgaroms/", 1, NULL);
     }
+    rom_reserve(0);
 
-    if (RomEnd == BUILD_ROM_START) {
+    if (rom_get_last() == BUILD_ROM_START)
         // No VGA rom found
-        RomEnd += OPTION_ROM_ALIGN;
         return;
-    }
 
+    VgaROM = (void*)BUILD_ROM_START;
     enable_vga_console();
 }
 
@@ -461,8 +463,7 @@ s3_resume_vga_init(void)
 {
     if (!S3ResumeVgaInit)
         return;
-    struct rom_header *rom = (void*)BUILD_ROM_START;
-    if (! is_valid_rom(rom))
+    if (!VgaROM || ! is_valid_rom(VgaROM))
         return;
-    callrom(rom, 0);
+    callrom(VgaROM, 0);
 }
